import bluebelt.styles.paper.summary
import bluebelt.styles.paper.control_chart
import bluebelt.styles.paper.run_chart
import bluebelt.styles.paper.process_capability
import bluebelt.styles.paper.patterns
import bluebelt.styles.paper.boxplot
import bluebelt.styles.paper.rolling_std
import bluebelt.styles.paper.correlation

import bluebelt.styles.paper.equal_means_plot
import bluebelt.styles.paper.equal_variances_plot
import bluebelt.styles.paper.distribution_plot
import bluebelt.styles.paper.resolution
import bluebelt.styles.paper.forecast
import bluebelt.styles.paper.planning

import bluebelt.styles.paper.graphs