import bluebelt.styles.defaults as defaults

# main plot | pyplot.plot
plot = {
    #'color': defaults.black,
    'linestyle': 'solid',
    'linewidth': 0,
    'marker': 'o',
    'markersize': 7,
    'markerfacecolor': defaults.blue,
    'markeredgewidth': 0,
    'markeredgecolor': defaults.blue,
    'alpha': 0.7,
    'zorder': 90,
}

# plot title | pyplot.set_title
title = {
    'loc': 'left',
}