import bluebelt.styles.paper.graphs.line
import bluebelt.styles.paper.graphs.scatter
import bluebelt.styles.paper.graphs.area
import bluebelt.styles.paper.graphs.hist
import bluebelt.styles.paper.graphs.boxplot
