import bluebelt.styles.defaults as defaults

# main plot | pyplot.plot
plot = {
    #'color': defaults.black,
    'linestyle': 'solid',
    'linewidth': 3,
    'zorder': 90,
}

multiindex_label_line = {
    'color': defaults.black,
}
# plot title | pyplot.set_title
title = {
    'loc': 'left',
}